@@ -54,10 +54,12 @@ import java.io.IOException;
 import java.nio.charset.UnsupportedCharsetException;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
+import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.Date;
 import java.util.Iterator;
+import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
 import java.util.Properties;
@@ -95,6 +97,7 @@ import com.openexchange.imap.entity2acl.Entity2ACLInit;
 import com.openexchange.imap.ping.IMAPCapabilityAndGreetingCache;
 import com.openexchange.imap.services.Services;
 import com.openexchange.imap.storecache.IMAPStoreCache;
+import com.openexchange.imap.storecache.IMAPStoreContainer;
 import com.openexchange.imap.util.HostAndPort;
 import com.openexchange.imap.util.HostAndPortAndCredentials;
 import com.openexchange.imap.util.StampAndOXException;
@@ -107,11 +110,14 @@ import com.openexchange.log.audit.DefaultAttribute.Name;
 import com.openexchange.mail.MailExceptionCode;
 import com.openexchange.mail.Protocol;
 import com.openexchange.mail.api.AuthType;
+import com.openexchange.mail.api.IMailFolderStorage;
+import com.openexchange.mail.api.IMailMessageStorage;
 import com.openexchange.mail.api.IMailProperties;
 import com.openexchange.mail.api.IMailStoreAware;
 import com.openexchange.mail.api.MailAccess;
 import com.openexchange.mail.api.MailConfig;
 import com.openexchange.mail.api.MailLogicTools;
+import com.openexchange.mail.cache.IMailAccessCache;
 import com.openexchange.mail.config.MailProperties;
 import com.openexchange.mail.dataobjects.MailFolder;
 import com.openexchange.mail.mime.MimeMailException;
@@ -135,6 +141,7 @@ import com.sun.mail.imap.GreetingListener;
 import com.sun.mail.imap.IMAPFolder;
 import com.sun.mail.imap.IMAPStore;
 import com.sun.mail.imap.JavaIMAPStore;
+import com.sun.mail.imap.Rights;
 
 /**
  * {@link IMAPAccess} - Establishes an IMAP access and provides access to storages.
@@ -589,7 +596,8 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
                  * Check read access
                  */
                 final ACLExtension aclExtension = imapConfig.getACLExtension();
-                if (!aclExtension.aclSupport() || aclExtension.canRead(IMAPFolderConverter.getOwnRights(imapFolder, session, imapConfig))) {
+                Rights ownRights = IMAPFolderConverter.getOwnRights(imapFolder, session, imapConfig);
+                if (!aclExtension.aclSupport() || (ownRights != null && aclExtension.canRead(ownRights))) {
                     retval = IMAPFolderConverter.getUnreadCount(imapFolder);
                 } else {
                     // ACL support AND no read access
@@ -664,7 +672,10 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
                 throw MimeMailException.handleMessagingException(e, config, session);
             } catch (final MessagingException e) {
                 if (MimeMailException.isSSLHandshakeException(e)) {
-                    OXException oxe = SSLExceptionCode.UNTRUSTED_CERTIFICATE.create(e.getCause(), config.getServer());
+                    List<Object> displayArgs = new ArrayList<>(2);
+                    displayArgs.add(SSLExceptionCode.extractArgument(e, "fingerprint"));
+                    displayArgs.add(config.getServer());
+                    OXException oxe = SSLExceptionCode.UNTRUSTED_CERTIFICATE.create(e.getCause(), displayArgs.toArray(new Object[] {}));
                     warnings.add(oxe);
                     throw oxe;
                 }
@@ -828,7 +839,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
                 /*
                  * Check for a SocketTimeoutException
                  */
-                if (tmpDownEnabled && MimeMailException.isTimeoutException(e)) {
+                if (tmpDownEnabled && MimeMailException.isTimeoutOrConnectException(e)) {
                     /*
                      * Remember a timed-out IMAP server on connect attempt
                      */
@@ -838,7 +849,10 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
                     }
                 }
                 if (MimeMailException.isSSLHandshakeException(e)) {
-                    throw SSLExceptionCode.UNTRUSTED_CERTIFICATE.create(e.getCause(), server);
+                    List<Object> displayArgs = new ArrayList<>(2);
+                    displayArgs.add(SSLExceptionCode.extractArgument(e, "fingerprint"));
+                    displayArgs.add(server);
+                    throw SSLExceptionCode.UNTRUSTED_CERTIFICATE.create(e.getCause(), displayArgs.toArray(new Object[] {}));
                 }
                 {
                     Exception next = e.getNextException();
@@ -923,6 +937,25 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
         return connectIMAPStore(maxCount, imapSession, server, port, login, password, clientIp);
     }
 
+    /**
+     * Clears cached IMAP connections.
+     */
+    protected void clearCachedConnections() {
+        final IMAPStoreContainer container = IMAPStoreCache.getInstance().optContainer(accountId, server, port, login, session);
+        if (null != container) {
+            container.clear();
+        }
+        try {
+            final IMailAccessCache mailAccessCache = MailAccess.getMailAccessCache();
+            MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> tmp;
+            while ((tmp = mailAccessCache.removeMailAccess(session, accountId)) != null) {
+                tmp.close(false);
+            }
+        } catch (final Exception e) {
+            // Ignore
+        }
+    }
+
     private static final String PROTOCOL = IMAPProvider.PROTOCOL_IMAP.getName();
 
     private IMAPStore connectIMAPStore(final int maxCount, final javax.mail.Session imapSession, final String server, final int port, final String login, final String pw, final String clientIp) throws MessagingException, OXException {
@@ -1537,12 +1570,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
         }
     }
 
-    /**
-     * (Safely) closes given IMAP store instance
-     *
-     * @param imapStore The IMAP store to close
-     */
-    public static void closeSafely(final IMAPStore imapStore) {
+    private static void closeSafely(final IMAPStore imapStore) {
         if (null != imapStore) {
             try {
                 imapStore.close();
