@@ -51,11 +51,10 @@ package com.openexchange.imap;
 
 import java.net.InetAddress;
 import java.net.UnknownHostException;
-import java.security.SecureRandom;
+import java.security.MessageDigest;
+import java.security.NoSuchAlgorithmException;
 import java.util.LinkedHashMap;
 import java.util.Map;
-import java.util.Random;
-import org.apache.commons.lang.RandomStringUtils;
 import com.openexchange.java.Strings;
 import com.openexchange.log.LogProperties;
 import com.openexchange.session.Session;
@@ -105,8 +104,6 @@ public enum IMAPClientParameters {
 
     // --------------------------------------------------------------------------------------------------------------------
 
-    private static final Random RANDOM = new SecureRandom();
-
     /**
      * Generates the session information.
      * <pre>
@@ -124,10 +121,70 @@ public enum IMAPClientParameters {
         buf.append(session.getSessionID());
         buf.append('-').append(session.getUserId());
         buf.append('-').append(session.getContextId());
-        buf.append('-').append(RandomStringUtils.random(12, 0, 0, true, true, null, RANDOM));
+        buf.append('-').append(getHashFor(session));
         return buf.toString();
     }
 
+    private static final java.util.concurrent.atomic.AtomicLong NEXT = new java.util.concurrent.atomic.AtomicLong(0);
+    private static final byte[] SERVER_ID = intToBytes(com.openexchange.exception.OXException.getServerId());
+
+    private static String getHashFor(Session session) {
+        try {
+            MessageDigest md = MessageDigest.getInstance("MD5");
+
+            long current;
+            long next;
+            do {
+                current = NEXT.get();
+                next = current + 1;
+                if (next < 0) {
+                    next = 0;
+                }
+            } while (false == NEXT.compareAndSet(current, next));
+            md.update(longToBytes(current));
+            md.update(SERVER_ID); // Node-unique salt
+
+            return asHex(md.digest(), 8);
+        } catch (NoSuchAlgorithmException e) {
+            // Ignore
+        }
+
+        return new StringBuilder(16).append(session.hashCode()).append(session.getUserId()).append(session.getContextId()).append(System.currentTimeMillis()).toString();
+    }
+
+    private static final char[] HEX_CHARS = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };
+
+    /**
+     * Turns array of bytes into string representing each byte as unsigned hex number.
+     *
+     * @param hash Array of bytes to convert to hex-string
+     * @return Generated hex string
+     */
+    private static String asHex(byte[] hash, int len) {
+        int length = len <= 0 || len > hash.length ?  hash.length : len;
+
+        char[] buf = new char[length << 1];
+        for (int i = 0, x = 0; i < length; i++) {
+            buf[x++] = HEX_CHARS[(hash[i] >>> 4) & 0xf];
+            buf[x++] = HEX_CHARS[hash[i] & 0xf];
+        }
+        return new String(buf);
+    }
+
+    private static byte[] longToBytes(long value) {
+        byte[] result = new byte[8];
+        long l = value;
+        for (int i = 7; i >= 0; i--) {
+            result[i] = (byte) (l & 0xFF);
+            l >>= 8;
+        }
+        return result;
+    }
+
+    private static byte[] intToBytes(int value) {
+        return new byte[] { (byte) (value >>> 24), (byte) (value >>> 16), (byte) (value >>> 8), (byte) value };
+    }
+
     /**
      * Generates the session information.
      * <pre>
