@@ -68,6 +68,7 @@ import javax.mail.FetchProfile;
 import javax.mail.Message;
 import javax.mail.MessagingException;
 import javax.mail.search.SearchException;
+import com.google.common.collect.ImmutableList;
 import com.openexchange.config.ConfigurationService;
 import com.openexchange.config.Interests;
 import com.openexchange.config.Reloadable;
@@ -98,16 +99,19 @@ import com.openexchange.mail.OrderDirection;
 import com.openexchange.mail.api.IMailFolderStorage;
 import com.openexchange.mail.api.IMailMessageStorage;
 import com.openexchange.mail.api.MailAccess;
+import com.openexchange.mail.api.MailConfig;
 import com.openexchange.mail.dataobjects.MailMessage;
 import com.openexchange.mail.dataobjects.MailThread;
 import com.openexchange.mail.dataobjects.ThreadSortMailMessage;
 import com.openexchange.mail.mime.ExtendedMimeMessage;
 import com.openexchange.mail.search.SearchTerm;
 import com.openexchange.mail.utils.MailMessageComparator;
+import com.openexchange.mail.utils.MailMessageComparatorFactory;
 import com.openexchange.mailaccount.MailAccount;
 import com.openexchange.session.Session;
 import com.openexchange.threadpool.AbstractTask;
 import com.openexchange.threadpool.ThreadPools;
+import com.openexchange.threadpool.behavior.CallerRunsBehavior;
 import com.sun.mail.iap.Argument;
 import com.sun.mail.iap.BadCommandException;
 import com.sun.mail.iap.ProtocolException;
@@ -331,11 +335,11 @@ public final class IMAPConversationWorker {
             long uidNext = imapMessageStorage.getImapFolder().getUIDNext();
             FetchProfile fp;
             if (byEnvelope) {
-                fp = Conversations.getFetchProfileConversationByEnvelope(null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
+                fp = Conversations.getFetchProfileConversationByEnvelope(imapMessageStorage.examineHasAttachmentUserFlags, null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
             } else {
-                fp = Conversations.getFetchProfileConversationByHeaders(null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
+                fp = Conversations.getFetchProfileConversationByHeaders(imapMessageStorage.examineHasAttachmentUserFlags, null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
             }
-            conversations = Conversations.conversationsFor(imapMessageStorage.getImapFolder(), lookAhead, order, fp, imapMessageStorage.getImapServerInfo(), byEnvelope);
+            conversations = Conversations.conversationsFor(imapMessageStorage.getImapFolder(), lookAhead, order, fp, imapMessageStorage.getImapServerInfo(), byEnvelope, imapMessageStorage.examineHasAttachmentUserFlags);
             if (conversations.isEmpty()) {
                 return Collections.emptyList();
             }
@@ -348,7 +352,7 @@ public final class IMAPConversationWorker {
                 sentTotal = imapMessageStorage.getImapFolder().getMessageCount();
                 sentUidNext = imapMessageStorage.getImapFolder().getUIDNext();
                 // Get sent messages
-                List<MailMessage> sentMessages = Conversations.messagesFor(imapMessageStorage.getImapFolder(), lookAhead, order, fp, imapMessageStorage.getImapServerInfo(), byEnvelope);
+                List<MailMessage> sentMessages = Conversations.messagesFor(imapMessageStorage.getImapFolder(), lookAhead, order, fp, imapMessageStorage.getImapServerInfo(), byEnvelope, imapMessageStorage.examineHasAttachmentUserFlags);
                 if (false == sentMessages.isEmpty()) {
                     // Filter messages already contained in conversations
                     {
@@ -367,8 +371,35 @@ public final class IMAPConversationWorker {
 
                     if (false == sentMessages.isEmpty()) {
                         // Add to conversation if references or referenced-by
-                        Conversations.foldAndMergeWithList(conversations, sentMessages);
+                        final List<MailMessage> toMergeWith = ImmutableList.copyOf(sentMessages);
                         sentMessages = null;
+                        boolean hasNext = true;
+                        for (Iterator<Conversation> iter = conversations.iterator(); hasNext;) {
+                            final Conversation conversation = iter.next();
+                            hasNext = iter.hasNext();
+                            if (hasNext) {
+                                // Not the last one...
+                                AbstractTask<Void> merge = new AbstractTask<Void>() {
+
+                                    @Override
+                                    public Void call() throws Exception {
+                                        for (MailMessage sentMessage : toMergeWith) {
+                                            if (conversation.referencesOrIsReferencedBy(sentMessage)) {
+                                                conversation.addMessage(sentMessage);
+                                            }
+                                        }
+                                        return null;
+                                    }
+                                };
+                                ThreadPools.getThreadPool().submit(merge, CallerRunsBehavior.<Void> getInstance());
+                            } else {
+                                for (MailMessage sentMessage : toMergeWith) {
+                                    if (conversation.referencesOrIsReferencedBy(sentMessage)) {
+                                        conversation.addMessage(sentMessage);
+                                    }
+                                }
+                            }
+                        }
                     }
                 }
                 // Switch back folder
@@ -471,10 +502,10 @@ public final class IMAPConversationWorker {
                         IMAPStore imapStore = messageStorage.getImapStore();
 
                         if (null != first) {
-                            fillMessagesStatic(first, fullName, sentFullName, mergeWithSent, usedFields, headerNames, isRev1, imapStore, imapMessageStorage.getImapServerInfo(), mailAccount);
+                            fillMessagesStatic(first, fullName, sentFullName, mergeWithSent, usedFields, headerNames, isRev1, imapStore, imapMessageStorage.getImapServerInfo(), mailAccount, imapFolderStorage.getImapConfig());
                         }
                         if (null != rest) {
-                            fillMessagesStatic(rest, fullName, sentFullName, mergeWithSent, usedFields, headerNames, isRev1, imapStore, imapMessageStorage.getImapServerInfo(), mailAccount);
+                            fillMessagesStatic(rest, fullName, sentFullName, mergeWithSent, usedFields, headerNames, isRev1, imapStore, imapMessageStorage.getImapServerInfo(), mailAccount, imapFolderStorage.getImapConfig());
                         }
                     } finally {
                         if (null != mailAccess) {
@@ -567,12 +598,13 @@ public final class IMAPConversationWorker {
         return list;
     }
 
-    static void fillMessagesStatic(List<List<MailMessage>> list, String fullName, String sentFullName, boolean mergeWithSent, MailFields usedFields, String[] headerNames, boolean isRev1, IMAPStore imapStore, IMAPServerInfo imapServerInfo, MailAccount mailAccount) throws MessagingException, OXException {
+    static void fillMessagesStatic(List<List<MailMessage>> list, String fullName, String sentFullName, boolean mergeWithSent, MailFields usedFields, String[] headerNames, boolean isRev1, IMAPStore imapStore, IMAPServerInfo imapServerInfo, MailAccount mailAccount, MailConfig mailConfig) throws MessagingException, OXException {
         IMAPFolder imapFolder = (IMAPFolder) imapStore.getFolder(fullName);
         imapFolder.open(IMAPFolder.READ_ONLY);
         try {
+            boolean examineHasAttachmentUserFlags = mailConfig.getCapabilities().hasAttachmentMarker();
             if (mergeWithSent) {
-                FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), headerNames, null, null, true));
+                FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), headerNames, null, null, true, examineHasAttachmentUserFlags));
                 List<MailMessage> msgs = new LinkedList<>();
                 List<MailMessage> sentmsgs = new LinkedList<>();
                 for (List<MailMessage> conversation : list) {
@@ -584,20 +616,20 @@ public final class IMAPConversationWorker {
                         }
                     }
                 }
-                new MailMessageFillerIMAPCommand(msgs, isRev1, fetchProfile, imapServerInfo, imapFolder).doCommand();
+                new MailMessageFillerIMAPCommand(msgs, isRev1, fetchProfile, imapServerInfo, examineHasAttachmentUserFlags, imapFolder).doCommand();
                 if (!sentmsgs.isEmpty()) {
                     // Switch folder
                     imapFolder.close(false);
                     imapFolder = (IMAPFolder) imapStore.getFolder(sentFullName);
                     imapFolder.open(IMAPFolder.READ_ONLY);
-                    new MailMessageFillerIMAPCommand(sentmsgs, isRev1, fetchProfile, imapServerInfo, imapFolder).doCommand();
+                    new MailMessageFillerIMAPCommand(sentmsgs, isRev1, fetchProfile, imapServerInfo, examineHasAttachmentUserFlags, imapFolder).doCommand();
                 }
             } else {
                 List<MailMessage> msgs = new LinkedList<>();
                 for (List<MailMessage> conversation : list) {
                     msgs.addAll(conversation);
                 }
-                new MailMessageFillerIMAPCommand(msgs, isRev1, getFetchProfile(usedFields.toArray(), headerNames, null, null, true), imapServerInfo, imapFolder).doCommand();
+                new MailMessageFillerIMAPCommand(msgs, isRev1, getFetchProfile(usedFields.toArray(), headerNames, null, null, true, examineHasAttachmentUserFlags), imapServerInfo, examineHasAttachmentUserFlags, imapFolder).doCommand();
             }
             /*
              * Apply account identifier
@@ -610,8 +642,9 @@ public final class IMAPConversationWorker {
 
     private void fillMessages(List<List<MailMessage>> list, String fullName, String sentFullName, boolean mergeWithSent, MailFields usedFields, String[] headerNames, boolean body, boolean isRev1) throws MessagingException, OXException {
         // Fill messages
+        boolean examineHasAttachmentUserFlags = imapMessageStorage.examineHasAttachmentUserFlags;
         if (mergeWithSent) {
-            FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), headerNames, null, null, true));
+            FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), headerNames, null, null, true, examineHasAttachmentUserFlags));
             List<MailMessage> msgs = new LinkedList<>();
             List<MailMessage> sentmsgs = new LinkedList<>();
             for (List<MailMessage> conversation : list) {
@@ -623,11 +656,11 @@ public final class IMAPConversationWorker {
                     }
                 }
             }
-            new MailMessageFillerIMAPCommand(msgs, isRev1, fetchProfile, imapMessageStorage.getImapServerInfo(), imapMessageStorage.getImapFolder()).doCommand();
+            new MailMessageFillerIMAPCommand(msgs, isRev1, fetchProfile, imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, imapMessageStorage.getImapFolder()).doCommand();
             if (!sentmsgs.isEmpty()) {
                 // Switch folder
                 imapMessageStorage.openReadOnly(sentFullName);
-                new MailMessageFillerIMAPCommand(sentmsgs, isRev1, fetchProfile, imapMessageStorage.getImapServerInfo(), imapMessageStorage.getImapFolder()).doCommand();
+                new MailMessageFillerIMAPCommand(sentmsgs, isRev1, fetchProfile, imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, imapMessageStorage.getImapFolder()).doCommand();
                 // Switch back folder
                 imapMessageStorage.openReadOnly(fullName);
             }
@@ -647,7 +680,7 @@ public final class IMAPConversationWorker {
                 for (List<MailMessage> conversation : list) {
                     msgs.addAll(conversation);
                 }
-                new MailMessageFillerIMAPCommand(msgs, isRev1, getFetchProfile(usedFields.toArray(), headerNames, null, null, true), imapMessageStorage.getImapServerInfo(), imapMessageStorage.getImapFolder()).doCommand();
+                new MailMessageFillerIMAPCommand(msgs, isRev1, getFetchProfile(usedFields.toArray(), headerNames, null, null, true, examineHasAttachmentUserFlags), imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, imapMessageStorage.getImapFolder()).doCommand();
             }
         }
         /*
@@ -669,41 +702,27 @@ public final class IMAPConversationWorker {
         }
         final boolean byEnvelope = false;
         final boolean isRev1 = imapMessageStorage.getImapConfig().getImapCapabilities().hasIMAP4rev1();
+        final boolean examineHasAttachmentUserFlags = imapMessageStorage.examineHasAttachmentUserFlags;
 
         List<List<MailMessage>> list;
         if (mergeWithSent) {
             FetchProfile fp;
             if (byEnvelope) {
-                fp = Conversations.getFetchProfileConversationByEnvelope(null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
+                fp = Conversations.getFetchProfileConversationByEnvelope(imapMessageStorage.examineHasAttachmentUserFlags, null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
             } else {
-                fp = Conversations.getFetchProfileConversationByHeaders(null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
+                fp = Conversations.getFetchProfileConversationByHeaders(imapMessageStorage.examineHasAttachmentUserFlags, null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
             }
-            List<Conversation> conversations = ThreadSorts.getConversationList(imapMessageStorage.getImapFolder(), getSortRange(lookAhead, messageCount, order), isRev1, fp, imapMessageStorage.getImapServerInfo(), searchTerm);
+            List<Conversation> conversations = ThreadSorts.getConversationList(imapMessageStorage.getImapFolder(), getSortRange(lookAhead, messageCount, order), isRev1, fp, imapMessageStorage.getImapServerInfo(), imapMessageStorage.examineHasAttachmentUserFlags, searchTerm);
             // Merge with sent folder
             {
                 // Switch folder
                 imapMessageStorage.openReadOnly(sentFullName);
-                List<MailMessage> sentMessages = Conversations.messagesFor(imapMessageStorage.getImapFolder(), lookAhead, order, fp, imapMessageStorage.getImapServerInfo(), byEnvelope);
-                if (false == sentMessages.isEmpty()) {
-                    // Filter messages already contained in conversations
-                    {
-                        Set<String> allMessageIds = new HashSet<>(conversations.size());
-                        for (final Conversation conversation : conversations) {
-                            conversation.addMessageIdsTo(allMessageIds);
+                final List<MailMessage> sentMessages = Conversations.messagesFor(imapMessageStorage.getImapFolder(), lookAhead, order, fp, imapMessageStorage.getImapServerInfo(), byEnvelope, examineHasAttachmentUserFlags);
+                for (final Conversation conversation : conversations) {
+                    for (final MailMessage sentMessage : sentMessages) {
+                        if (conversation.referencesOrIsReferencedBy(sentMessage)) {
+                            conversation.addMessage(sentMessage);
                         }
-                        for (Iterator<MailMessage> iter = sentMessages.iterator(); iter.hasNext();) {
-                            MailMessage sentMessage = iter.next();
-                            if (allMessageIds.contains(sentMessage.getMessageId())) {
-                                // Already contained
-                                iter.remove();
-                            }
-                        }
-                    }
-
-                    if (false == sentMessages.isEmpty()) {
-                        // Add to conversation if references or referenced-by
-                        Conversations.foldAndMergeWithList(conversations, sentMessages);
-                        sentMessages = null;
                     }
                 }
                 // Switch back folder
@@ -739,7 +758,7 @@ public final class IMAPConversationWorker {
             }
             // Fill selected chunk
             if (!list.isEmpty()) {
-                FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), headerNames, null, null, true));
+                FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), headerNames, null, null, true, examineHasAttachmentUserFlags));
                 List<MailMessage> msgs = new LinkedList<>();
                 List<MailMessage> sentmsgs = new LinkedList<>();
                 for (List<MailMessage> conversation : list) {
@@ -751,15 +770,15 @@ public final class IMAPConversationWorker {
                         }
                     }
                 }
-                new MailMessageFillerIMAPCommand(msgs, isRev1, fetchProfile, imapMessageStorage.getImapServerInfo(), imapMessageStorage.getImapFolder()).doCommand();
+                new MailMessageFillerIMAPCommand(msgs, isRev1, fetchProfile, imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, imapMessageStorage.getImapFolder()).doCommand();
                 if (!sentmsgs.isEmpty()) {
                     // Switch folder
                     imapMessageStorage.openReadOnly(sentFullName);
-                    new MailMessageFillerIMAPCommand(sentmsgs, isRev1, fetchProfile, imapMessageStorage.getImapServerInfo(), imapMessageStorage.getImapFolder()).doCommand();
+                    new MailMessageFillerIMAPCommand(sentmsgs, isRev1, fetchProfile, imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, imapMessageStorage.getImapFolder()).doCommand();
                 }
             }
         } else {
-            list = ThreadSorts.getConversations(imapMessageStorage.getImapFolder(), getSortRange(lookAhead, messageCount, order), isRev1, imapMessageStorage.getImapServerInfo(), searchTerm, null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
+            list = ThreadSorts.getConversations(imapMessageStorage.getImapFolder(), getSortRange(lookAhead, messageCount, order), isRev1, imapMessageStorage.examineHasAttachmentUserFlags, imapMessageStorage.getImapServerInfo(), searchTerm, imapFolderStorage.getImapConfig(), null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
             // Sort root elements
             {
                 final MailSortField effectiveSortField = null == sortField ? MailSortField.RECEIVED_DATE : sortField;
@@ -797,7 +816,7 @@ public final class IMAPConversationWorker {
                 for (List<MailMessage> conversation : list) {
                     msgs.addAll(conversation);
                 }
-                new MailMessageFillerIMAPCommand(msgs, isRev1, getFetchProfile(usedFields.toArray(), headerNames, null, null, true), imapMessageStorage.getImapServerInfo(), imapMessageStorage.getImapFolder()).doCommand();
+                new MailMessageFillerIMAPCommand(msgs, isRev1, getFetchProfile(usedFields.toArray(), headerNames, null, null, true, imapFolderStorage.examineHasAttachmentUserFlags), imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, imapMessageStorage.getImapFolder()).doCommand();
             }
         }
         // Apply account identifier
@@ -821,7 +840,7 @@ public final class IMAPConversationWorker {
     }
 
     private Comparator<List<MailMessage>> getListComparator(final MailSortField sortField, final OrderDirection order, final String fullName, final Locale locale) {
-        final MailMessageComparator comparator = new MailMessageComparator(sortField, OrderDirection.DESC.equals(order), locale, imapFolderStorage.getImapConfig().getIMAPProperties().isUserFlagsEnabled());
+        final MailMessageComparator comparator = MailMessageComparatorFactory.createComparator(sortField, order, locale, imapFolderStorage.getSession(), imapFolderStorage.getImapConfig().getIMAPProperties().isUserFlagsEnabled());
         Comparator<List<MailMessage>> listComparator = new Comparator<List<MailMessage>>() {
 
             @Override
@@ -967,8 +986,16 @@ public final class IMAPConversationWorker {
 
         @Override
         public Object doCommand(IMAPProtocol protocol) throws ProtocolException {
-            if (!protocol.hasCapability("THREAD=REFERENCES")) {
-                throw new BadCommandException("THREAD=REFERENCES not supported");
+            String algorithm;
+            if (protocol.hasCapability("THREAD=REFS")) {
+                // Prefer "REFS" algorithm, which does the threading using only References/In-Reply-To headers
+                algorithm = "REFS";
+            } else {
+                if (!protocol.hasCapability("THREAD=REFERENCES")) {
+                    throw new BadCommandException("THREAD=REFERENCES not supported");
+                }
+                // Considers both - References/In-Reply-To headers and Subject - when doing the threading
+                algorithm = "REFERENCES";
             }
             if (searchKeys == null || searchKeys.length == 0) {
                 throw new BadCommandException("Must have at least one sort term");
@@ -978,7 +1005,7 @@ public final class IMAPConversationWorker {
 
             Argument args = new Argument();
 
-            args.writeAtom("REFERENCES");    // charset specification
+            args.writeAtom(algorithm);    // algorithm specification
             args.writeAtom("UTF-8");    // charset specification
 
             // Add (sort) terms
@@ -1030,10 +1057,10 @@ public final class IMAPConversationWorker {
 
         // Fill requested fields/headers
         {
-            FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), headerNames, null, null, true));
+            FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), headerNames, null, null, true, imapMessageStorage.examineHasAttachmentUserFlags));
             boolean isRev1 = imapMessageStorage.getImapConfig().getImapCapabilities().hasIMAP4rev1();
             TLongObjectMap<MailMessage> messages = parseResult.getMessages();
-            new MailMessageFillerIMAPCommand(messages.valueCollection(), isRev1, fetchProfile, imapMessageStorage.getImapServerInfo(), imapMessageStorage.getImapFolder()).doCommand();
+            new MailMessageFillerIMAPCommand(messages.valueCollection(), isRev1, fetchProfile, imapMessageStorage.getImapServerInfo(), imapMessageStorage.examineHasAttachmentUserFlags, imapMessageStorage.getImapFolder()).doCommand();
             imapMessageStorage.setAccountInfo(messages.values(new MailMessage[messages.size()]));
         }
 
@@ -1048,7 +1075,7 @@ public final class IMAPConversationWorker {
     }
 
     private Comparator<MailThread> getThreadComparator(final MailSortField sortField, final OrderDirection order, Locale locale) {
-        final MailMessageComparator comparator = new MailMessageComparator(sortField, OrderDirection.DESC.equals(order), locale, imapFolderStorage.getImapConfig().getIMAPProperties().isUserFlagsEnabled());
+        final MailMessageComparator comparator = MailMessageComparatorFactory.createComparator(sortField, order, locale, imapFolderStorage.getSession(), imapFolderStorage.getImapConfig().getIMAPProperties().isUserFlagsEnabled());
         Comparator<MailThread> threadComparator = new Comparator<MailThread>() {
 
             @Override
@@ -1099,7 +1126,6 @@ public final class IMAPConversationWorker {
                 }
             }
             final MailSortField effectiveSortField = null == sortField ? MailSortField.RECEIVED_DATE : sortField;
-            final boolean descending = OrderDirection.DESC.equals(order);
             /*
              * Create threaded structure dependent on THREAD=REFERENCES capability
              */
@@ -1157,12 +1183,12 @@ public final class IMAPConversationWorker {
             usedFields.add(MailField.THREAD_LEVEL);
             // Add sort field
             usedFields.add(MailField.toField(effectiveSortField.getListField()));
-            final FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), imapMessageStorage.getIMAPProperties().isFastFetch()));
+            final FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), imapMessageStorage.getIMAPProperties().isFastFetch(), imapMessageStorage.examineHasAttachmentUserFlags));
             final boolean body = usedFields.contains(MailField.BODY) || usedFields.contains(MailField.FULL);
             if (!body) {
                 final Map<MessageInfo, MailMessage> mapping;
                 {
-                    TLongObjectMap<MailMessage> messages = new SimpleFetchIMAPCommand(imapMessageStorage.getImapFolder(), imapMessageStorage.getImapConfig().getImapCapabilities().hasIMAP4rev1(), MessageInfo.toSeqNums(messageIds).toArray(), fetchProfile, imapMessageStorage.getImapServerInfo()).doCommand();
+                    TLongObjectMap<MailMessage> messages = new SimpleFetchIMAPCommand(imapMessageStorage.getImapFolder(), imapMessageStorage.getImapConfig().getImapCapabilities().hasIMAP4rev1(), MessageInfo.toSeqNums(messageIds).toArray(), fetchProfile, imapMessageStorage.getImapServerInfo(), imapMessageStorage.examineHasAttachmentUserFlags).doCommand();
                     mapping = new HashMap<>(messages.size());
                     messages.forEachEntry(new TLongObjectProcedure<MailMessage>() {
 
@@ -1177,7 +1203,8 @@ public final class IMAPConversationWorker {
                 /*
                  * Sort according to order direction
                  */
-                Collections.sort(structuredList, new MailMessageComparator(effectiveSortField, descending, imapMessageStorage.getLocale(), imapMessageStorage.getIMAPProperties().isUserFlagsEnabled()));
+                Collections.sort(structuredList, MailMessageComparatorFactory.createComparator(effectiveSortField, order, imapMessageStorage.getLocale(), imapFolderStorage.getSession(), imapMessageStorage.getIMAPProperties().isUserFlagsEnabled()));
+
                 /*
                  * Output as flat list
                  */
@@ -1258,7 +1285,8 @@ public final class IMAPConversationWorker {
             /*
              * Sort according to order direction
              */
-            Collections.sort(structuredList, new MailMessageComparator(effectiveSortField, descending, imapMessageStorage.getLocale(), imapMessageStorage.getIMAPProperties().isUserFlagsEnabled()));
+            Collections.sort(structuredList, MailMessageComparatorFactory.createComparator(effectiveSortField, order, imapMessageStorage.getLocale(), imapFolderStorage.getSession(), imapMessageStorage.getIMAPProperties().isUserFlagsEnabled()));
+
             /*
              * Output as flat list
              */
