@@ -55,10 +55,8 @@ import static com.openexchange.imap.util.ImapUtility.prepareImapCommandForLoggin
 import static com.openexchange.mail.MailServletInterface.mailInterfaceMonitor;
 import java.util.ArrayList;
 import java.util.Collections;
-import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
-import java.util.Map;
 import javax.mail.FetchProfile;
 import javax.mail.FetchProfile.Item;
 import javax.mail.MessagingException;
@@ -84,7 +82,6 @@ import com.sun.mail.imap.IMAPFolder;
 import com.sun.mail.imap.protocol.FetchResponse;
 import com.sun.mail.imap.protocol.IMAPProtocol;
 
-
 /**
  * {@link Conversations} - Utility class.
  *
@@ -124,12 +121,12 @@ public final class Conversations {
      * @param fields The fields to add
      * @return The <i>"by envelope"</i> fetch profile
      */
-    public static FetchProfile getFetchProfileConversationByEnvelope(MailField... fields) {
+    public static FetchProfile getFetchProfileConversationByEnvelope(boolean considerUserFlags, MailField... fields) {
         FetchProfile fp = newFetchProfile(true);
         if (null != fields) {
             for (MailField field : fields) {
                 if (!MimeStorageUtility.isEnvelopeField(field)) {
-                    MimeStorageUtility.addFetchItem(fp, field);
+                    MimeStorageUtility.addFetchItem(fp, field, considerUserFlags);
                 }
             }
         }
@@ -142,14 +139,14 @@ public final class Conversations {
      * @param fields The fields to add
      * @return The <i>"by headers"</i> fetch profile
      */
-    public static FetchProfile getFetchProfileConversationByHeaders(MailField... fields) {
+    public static FetchProfile getFetchProfileConversationByHeaders(boolean considerUserFlags, MailField... fields) {
         FetchProfile fp = newFetchProfile(false);
         if (null != fields) {
             for (MailField field : fields) {
                 if (MailField.RECEIVED_DATE.equals(field)) {
                     fp.add(MailMessageFetchIMAPCommand.INTERNALDATE);
                 } else {
-                    MimeStorageUtility.addFetchItem(fp, field);
+                    MimeStorageUtility.addFetchItem(fp, field, considerUserFlags);
                 }
             }
         }
@@ -199,11 +196,12 @@ public final class Conversations {
      * @param fetchProfile The fetch profile
      * @param serverInfo The IMAP server information
      * @param byEnvelope Whether to build-up using ENVELOPE; otherwise <code>false</code>
+     * @param examineHasAttachmentUserFlags Whether has-attachment user flags should be considered
      * @return The unfolded conversations
      * @throws MessagingException If a messaging error occurs
      */
-    public static List<Conversation> conversationsFor(IMAPFolder imapFolder, int lookAhead, OrderDirection order, FetchProfile fetchProfile, IMAPServerInfo serverInfo, boolean byEnvelope) throws MessagingException {
-        final List<MailMessage> messages = messagesFor(imapFolder, lookAhead, order, fetchProfile, serverInfo, byEnvelope);
+    public static List<Conversation> conversationsFor(IMAPFolder imapFolder, int lookAhead, OrderDirection order, FetchProfile fetchProfile, IMAPServerInfo serverInfo, boolean byEnvelope, boolean examineHasAttachmentUserFlags) throws MessagingException {
+        final List<MailMessage> messages = messagesFor(imapFolder, lookAhead, order, fetchProfile, serverInfo, byEnvelope, examineHasAttachmentUserFlags);
         if (null == messages || messages.isEmpty()) {
             return Collections.<Conversation> emptyList();
         }
@@ -223,11 +221,12 @@ public final class Conversations {
      * @param fetchProfile The fetch profile
      * @param serverInfo The IMAP server information
      * @param byEnvelope Whether to build-up using ENVELOPE; otherwise <code>false</code>
+     * @param examineHasAttachmentUserFlags Hwther has-attachment user flags should be considered
      * @return The messages with conversation information (References, In-Reply-To, Message-Id)
      * @throws MessagingException If a messaging error occurs
      */
     @SuppressWarnings("unchecked")
-    public static List<MailMessage> messagesFor(final IMAPFolder imapFolder, final int lookAhead, final OrderDirection order, final FetchProfile fetchProfile, final IMAPServerInfo serverInfo, final boolean byEnvelope) throws MessagingException {
+    public static List<MailMessage> messagesFor(final IMAPFolder imapFolder, final int lookAhead, final OrderDirection order, final FetchProfile fetchProfile, final IMAPServerInfo serverInfo, final boolean byEnvelope, final boolean examineHasAttachmentUserFlags) throws MessagingException {
         final int messageCount = imapFolder.getMessageCount();
         if (messageCount <= 0) {
             /*
@@ -273,15 +272,15 @@ public final class Conversations {
                 final Response response = r[len];
                 if (response.isOK()) {
                     try {
-                        final List<MailMessage> mails = new ArrayList<MailMessage>(len);
+                        final List<MailMessage> mails = new ArrayList<MailMessage>(messageCount);
                         final String fullName = imapFolder.getFullName();
                         final String sInReplyTo = "In-Reply-To";
                         final String sReferences = "References";
                         for (int j = 0; j < len; j++) {
                             if (r[j] instanceof FetchResponse) {
-                                final MailMessage message = handleFetchRespone((FetchResponse) r[j], fullName, serverInfo.getAccountId());
-                                if (null == message.getFirstHeader(sReferences)) {
-                                    // No "References" header
+                                final MailMessage message = handleFetchRespone((FetchResponse) r[j], fullName, serverInfo.getAccountId(), examineHasAttachmentUserFlags);
+                                final String references = message.getFirstHeader(sReferences);
+                                if (null == references) {
                                     final String inReplyTo = message.getFirstHeader(sInReplyTo);
                                     if (null != inReplyTo) {
                                         message.setHeader(sReferences, inReplyTo);
@@ -358,78 +357,23 @@ public final class Conversations {
      * @return The folded conversations
      */
     public static List<Conversation> fold(final List<Conversation> toFold) {
-        fold(toFold, new HashMap<String, Conversation>(toFold.size()));
+        Iterator<Conversation> iter = toFold.iterator();
+        int i = 0;
+        while (iter.hasNext()) {
+            foldInto(iter.next(), iter);
+            iter = toFold.listIterator(++i);
+        }
         return toFold;
     }
 
-    private static void fold(final List<Conversation> toFold, Map<String, Conversation> lookupTable) {
-        for (Iterator<Conversation> iter = toFold.iterator(); iter.hasNext();) {
-            Conversation conversation = iter.next();
-
-            boolean removed = false;
-            for (String messageId : conversation.getMessageIds()) {
-                Conversation existing = lookupTable.get(messageId);
-                if (null != existing) {
-                    if (!conversation.equals(existing)) {
-                        if (!removed) {
-                            iter.remove();
-                            removed = true;
-                        }
-                        existing.join(conversation);
-                    }
-                } else {
-                    lookupTable.put(messageId, conversation);
-                }
-            }
-
-            for (String reference : conversation.getReferences()) {
-                Conversation existing = lookupTable.get(reference);
-                if (null != existing) {
-                    if (!conversation.equals(existing)) {
-                        if (!removed) {
-                            iter.remove();
-                            removed = true;
-                        }
-                        existing.join(conversation);
-                    }
-                } else {
-                    lookupTable.put(reference, conversation);
-                }
+    private static void foldInto(final Conversation conversation, final Iterator<Conversation> iter) {
+        while (iter.hasNext()) {
+            final Conversation other = iter.next();
+            if (conversation.referencesOrIsReferencedBy(other)) {
+                iter.remove();
+                conversation.join(other);
             }
         }
     }
 
-    /**
-     * Folds specified conversations and adds MaillMessages to the list.
-     *
-     * @param toFold The conversations to fold
-     * @param toMergeWith The List of MaillMessages to be added
-     * @return The folded conversations
-     */
-    public static List<Conversation> foldAndMergeWithList(final List<Conversation> toFold, List<MailMessage> toMergeWith) {
-        HashMap<String, Conversation> lookupTable = new HashMap<String, Conversation>(toFold.size());
-        fold(toFold, lookupTable);
-        for (MailMessage mailMessage : toMergeWith) {
-            String messageId = mailMessage.getMessageId();
-            if (null != messageId) {
-                Conversation conversation = lookupTable.get(messageId);
-                if (null != conversation) {
-                    conversation.addMessage(mailMessage);
-                }
-            }
-
-            String[] references = mailMessage.getReferences();
-            if (null != references) {
-                for (String reference : references) {
-                    Conversation conversation = lookupTable.get(reference);
-                    if (null != conversation) {
-                        conversation.addMessage(mailMessage);
-                    }
-                }
-            }
-
-        }
-        return toFold;
-    }
-
 }
