@@ -69,6 +69,7 @@ import com.openexchange.imap.threader.references.Conversation;
 import com.openexchange.imap.util.WrappingProtocolException;
 import com.openexchange.log.LogProperties;
 import com.openexchange.mail.MailField;
+import com.openexchange.mail.api.MailConfig;
 import com.openexchange.mail.dataobjects.IDMailMessage;
 import com.openexchange.mail.dataobjects.MailMessage;
 import com.openexchange.mail.mime.utils.MimeStorageUtility;
@@ -81,7 +82,6 @@ import com.sun.mail.imap.protocol.IMAPProtocol;
 import com.sun.mail.imap.protocol.IMAPResponse;
 import com.sun.mail.imap.protocol.SearchSequence;
 
-
 /**
  * {@link ThreadSorts}
  *
@@ -119,7 +119,7 @@ public final class ThreadSorts {
 
                 if (searchTerm != null) {
                     try {
-                        args.append(new SearchSequence().generateSequence(searchTerm.getJavaMailSearchTerm(), "UTF-8"));
+                        args.append(new SearchSequence(p).generateSequence(searchTerm.getJavaMailSearchTerm(), "UTF-8"));
                     } catch (final IOException ioex) {
                         // should never happen
                         throw new WrappingProtocolException("", new SearchException(ioex.toString()));
@@ -177,11 +177,12 @@ public final class ThreadSorts {
      * @param isRev1 Flag for IMAPrev1
      * @param serverInfo The IMAP server information
      * @param fields The optional fields to pre-fill
+     * @param mailConfig The mail configuration
      * @return The conversation listing
      * @throws OXException If parsing fails
      * @throws MessagingException If acquiring the THREAD response fails
      */
-    public static List<List<MailMessage>> getConversations(IMAPFolder imapFolder, String sortRange, boolean isRev1, IMAPServerInfo serverInfo, SearchTerm<?> searchTerm, MailField... fields) throws OXException, MessagingException {
+    public static List<List<MailMessage>> getConversations(IMAPFolder imapFolder, String sortRange, boolean isRev1, boolean examineHasAttachmentUserFlags, IMAPServerInfo serverInfo, SearchTerm<?> searchTerm, MailConfig mailConfig, MailField... fields) throws OXException, MessagingException {
         if (null == fields || fields.length == 0 || (fields.length == 1 && MailField.RECEIVED_DATE.equals(fields[0]))) {
             return parseConversations(getThreadResponse(imapFolder, sortRange, true, searchTerm), imapFolder.getFullName());
         }
@@ -189,9 +190,9 @@ public final class ThreadSorts {
         // Its fetch profile...
         FetchProfile fp = new FetchProfile();
         for (MailField field : fields) {
-            MimeStorageUtility.addFetchItem(fp, field);
+            MimeStorageUtility.addFetchItem(fp, field, examineHasAttachmentUserFlags);
         }
-        return getConversations(imapFolder, sortRange, isRev1, fp, serverInfo, searchTerm);
+        return getConversations(imapFolder, sortRange, isRev1, fp, serverInfo, mailConfig.getCapabilities().hasAttachmentMarker(), searchTerm);
     }
 
     /**
@@ -204,11 +205,12 @@ public final class ThreadSorts {
      * @param isRev1 Flag for IMAPrev1
      * @param fetchProfile The optional fetch profile to pre-fill messages
      * @param serverInfo The IMAP server information
+     * @param examineHasAttachmentUserFlags Whether has-attachment user flags should be considered
      * @return The conversation listing
      * @throws OXException If parsing fails
      * @throws MessagingException If acquiring the THREAD response fails
      */
-    public static List<List<MailMessage>> getConversations(final IMAPFolder imapFolder, final String sortRange, final boolean isRev1, final FetchProfile fetchProfile, IMAPServerInfo serverInfo, SearchTerm<?> searchTerm) throws OXException, MessagingException {
+    public static List<List<MailMessage>> getConversations(final IMAPFolder imapFolder, final String sortRange, final boolean isRev1, final FetchProfile fetchProfile, IMAPServerInfo serverInfo, boolean examineHasAttachmentUserFlags, SearchTerm<?> searchTerm) throws OXException, MessagingException {
         if (null == fetchProfile) {
             return parseConversations(getThreadResponse(imapFolder, sortRange, true, searchTerm), imapFolder.getFullName());
         }
@@ -224,7 +226,7 @@ public final class ThreadSorts {
                 }
             }
             // Fill them
-            new MailMessageFillerIMAPCommand(msgs, isRev1, fetchProfile, serverInfo, imapFolder).doCommand();
+            new MailMessageFillerIMAPCommand(msgs, isRev1, fetchProfile, serverInfo, examineHasAttachmentUserFlags, imapFolder).doCommand();
         }
 
         return conversations;
@@ -240,11 +242,12 @@ public final class ThreadSorts {
      * @param isRev1 Flag for IMAPrev1
      * @param fetchProfile The optional fetch profile to pre-fill messages
      * @param serverInfo The IMAP server information
+     * @param examineHasAttachmentUserFlags Whether has-attachment user flags should be considered
      * @return The conversation listing
      * @throws OXException If parsing fails
      * @throws MessagingException If acquiring the THREAD response fails
      */
-    public static List<Conversation> getConversationList(final IMAPFolder imapFolder, final String sortRange, final boolean isRev1, final FetchProfile fetchProfile, IMAPServerInfo serverInfo, SearchTerm<?> searchTerm) throws OXException, MessagingException {
+    public static List<Conversation> getConversationList(final IMAPFolder imapFolder, final String sortRange, final boolean isRev1, final FetchProfile fetchProfile, IMAPServerInfo serverInfo, boolean examineHasAttachmentUserFlags, SearchTerm<?> searchTerm) throws OXException, MessagingException {
         List<List<MailMessage>> conversations = parseConversations(getThreadResponse(imapFolder, sortRange, true, searchTerm), imapFolder.getFullName());
         {
             // Turn conversations to a flat list
@@ -255,7 +258,7 @@ public final class ThreadSorts {
                 }
             }
             // Fill them
-            new MailMessageFillerIMAPCommand(msgs, isRev1, fetchProfile, serverInfo, imapFolder).doCommand();
+            new MailMessageFillerIMAPCommand(msgs, isRev1, fetchProfile, serverInfo, examineHasAttachmentUserFlags, imapFolder).doCommand();
         }
 
         List<Conversation> retval = new ArrayList<>(conversations.size());
@@ -296,8 +299,7 @@ public final class ThreadSorts {
 
             // System.out.println(threadList.substring(off, end));
 
-            @SuppressWarnings("unchecked")
-            List<MailMessage> conversation = (List<MailMessage>) parseConversationList(threadList.substring(off, end), fullName);
+            @SuppressWarnings("unchecked") List<MailMessage> conversation = (List<MailMessage>) parseConversationList(threadList.substring(off, end), fullName);
             conversations.add(conversation);
 
             off = end;
